//------------------------------------------------------------------------------------------
// File : AsuraInterface.h
//
// Asura User Interface Library
//
// Date : May 27, 2008
// Version : 1.0
// Author : Pocol
//------------------------------------------------------------------------------------------

#ifndef _ASURA_INTERFACE_H_
#define _ASURA_INTERFACE_H_

// Disable Warning C4996
#ifndef _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES
#define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES 1
#endif
#ifndef _CRT_SECURE_CPP_OVERLOAD_SECURE_NAMES
#define _CRT_SECURE_CPP_OVERLOAD_SECURE_NAMES 1
#endif

//
// include
//
#include <iostream>
#include <GL/glut.h>
using namespace std;

#define UI_LINE_HEIGHT		(float)20.0f
#define UI_BOX_SIZE			(float)10.0f
#define UI_SLIDER_SIZE		(float)10.0f

namespace Asura
{
	enum MouseButtonState
	{
		MOUSE_BUTTON_DOWN = 0,
		MOUSE_BUTTON_UP,
	};

	struct Point
	{
		int x, y;
		Point() : x(0), y(0) {}
		Point(int nx, int ny) : x(nx), y(ny) {}
		Point(Point &p) : x(p.x), y(p.y) {}
		Point &operator = (Point &p)	{ x=p.x; y=p.y; return *this; }
		void Zero() { x=0; y=0; }
	};

	struct MouseButton
	{
		Point down;
		Point up;
		MouseButtonState state;
	};

	//////////////////////////////////////////////////////////////////////
	//  AsuraUserInterface class
	//////////////////////////////////////////////////////////////////////
	class AsuraUserInterface
	{
	protected:
		Point m_cursor;
		Point m_down;
		Point m_delta;
		Point m_up;
		Point m_windowSize;
		MouseButtonState m_state;
		MouseButtonState m_stateClone;
		float m_alpha;
		bool m_focus;

		GLboolean m_lightingEnable;
		GLboolean m_blendEnable;
		GLint m_blendSrc;
		GLint m_blendDst;

		int GetTextWidth(char *text);
		void DrawPartsText(int x, int y, char *text, int depth=0);
		void DrawPartsStaticText(int x, int y, char *text, int depth=0);
		void DrawPartsButton(int x, int y, int w, int h, int flag, int stroke=2);
		void DrawPartsHorizontalSlider(int x, int y, int w, float pos);
		void DrawPartsVerticalSlider(int x, int y, int h, float pos);
		void DrawPartsCheckBox(int x, int y, bool flag);
		void DrawPartsListBox(int x, int y, int w, int numItem, int curLine);
		void DrawPartsComboBox(int x, int y, int w, int numItem, int curIndex, int flag);
		void DrawPartsSpinControl(int x, int y, int w, int upflag, int downflag);
		void DrawPartsFrame(float x, float y, float w, float h, float depth=0.0f);
		void DrawPartsGroupBox(int x, int y, int w, int h, int textWidth);
		void DrawPartsGauge(int x, int y, int w, int h, int vlaue, float *color);

	public:
		AsuraUserInterface();
		~AsuraUserInterface();

		void SetAlpha( float alpha );
		void Begin();
		void End();
		void Mouse(int button, int state, int x, int y);
		void Motion(int x, int y);
		void PassiveMotion(int x, int y);

		void DoText(int x, int y, char *text);
		void DoStaticText(int x, int y, char *text);
		void DoTextBox(int x, int y, int w, int h, char *text);
		void DoButton(int x, int y, int w, int h, char *text, bool &state);
		void DoRadioButton(int x, int y, int index, char *text, int &curIndex);
		void DoHorizontalSlider(int x, int y, int w, float min, float max, float &position);
		void DoVerticalSlider(int x, int y, int h, float min, float max, float &position);
		void DoCheckBox(int x, int y, char *text, bool &state);
		void DoListBox(int x, int y, int w, int numItem, char *text[], int &curLine);
		void DoComboBox(int x, int y, int w, int numItem, char *text[], int &curIndex, bool &dispListBox);
		void DoSpinControl(int x, int y, int w, int min, int max, int &value, int count=1);
		void DoSpinControl(int x, int y, int w, double min, double max, double &value, double count=1.0);
		void DoSpinControl(int x, int y, int w, float min, float max, float &value, float count=1.0f);
		void DoGroupBox(int x, int y, int w, int h, char *caption);
		void DoGauage(int x, int y, int w, int h, int min, int max, int value, float* color); 
	};
}
#endif